 subtitle  "I2C - I2C Master Routines using RA3/RA4."
;
;  The Routines contained in this .inc file allow communication between a
;   PIC equipped with RA4 as an "Open Collector" output.  This code is designed
;   for the PIC to be the only Master on the I2C Bus.
;
;  This code is designed to work for 7 bit addressed I2C devices.  The PIC
;   speed is nominally 4 MHz.  For faster PIC clock speeds, a lower Pull-Up
;   value should be used.
;
;  At the start of the host software, RA4 and RA3 must be set to "Output"
;   (their respective "TRISB" bits set to "0").  The normal inactive state
;   consists of the SDA and SCL lines low.
;
;  During Transmit, the Acknowledge Lines are NOT checked to see if the Receiver
;   is active and accepting the incoming data.
;
;  A typical I2C EEPROM Read or two bytes consists of:
;
;   call  I2CStart              ;  Send the Start Bit
;
;   movlw 0x005                 ;  Send the EEPROM Device Enable
;   call  I2CSend
;
;   movf  Address, w            ;  Send the Address within the EEPROM
;   call  I2CSend
;
;   call  I2CStart              ;  Now, Going to Start Reading
;
;   movlw 0x085                 ;  Enable the EEPROM Read
;   call  I2CSend
;
;   clrw                        ;  Receive the 1st byte (and Acknowledge it)
;   call  I2CReceive
;   movwf Data1st
;
;   movlw 1                     ;  Receive the 2nd byte - Don't Ack it to Stop the Transfer
;   call  I2CReceive
;   movwf Data2nd
;
;   call  I2CStop               ;  Send the Stop bit to Stop the Transfer
;
;
;  Hardware Notes:
;   The Pic is the only Master on the I2C Bus
;   RA3 is Connected to "SCL" on the I2C Bus
;   RA4 is Connected to "SDA" on the I2C Bus (which has a 1 - 10K Pull-Up)
;   I2CTemp is a file Register
;
;  Myke Predko
;  97.02.18
;

I2CStart                        ;  Send the Start Bit on the I2C Bus

  bsf    PORTA, 4               ;  Make the Data Line High
  goto   $ + 1
  bcf    PORTA, 3               ;  Make the Clock Line Low
  nop
  bsf    PORTA, 3               ;  Set the Clock High
  goto   $ + 1                  ;   for 5 Clock Cycles
  goto   $ + 1
  nop
  bcf    PORTA, 4               ;  Drop the Clock Line to Go into Start Mode
  goto   $ + 1                  ;   for 5 Clock Cycles
  goto   $ + 1
  nop
  bcf    PORTA, 3               ;  Start Clock Train
  goto   $ + 1

  return


I2CStop                         ;  Send the Stop Bit to the I2C Bus

  bcf    PORTA, 4               ;  Data Line Low
  goto   $ + 1
  goto   $ + 1
  bsf    PORTA, 3               ;  Set the Clock Line High
  goto   $ + 1
  goto   $ + 1
  bsf    PORTA, 4               ;  Data goes high whil Clock is High for Stop Bit
  goto   $ + 1
  bcf    PORTA, 3               ;  Set Clock Low Again
  goto   $ + 1
  nop

  return


I2CSend                         ;  Send the Data Out

  movwf  I2CTemp                ;  Store the Value to Send

  movlw  8                      ;  Sending 8 Bits

I2CWLoop                        ;  Loop Around Here until Each Bit Sent

  rrf    I2CTemp                ;  Figure Out what we have
  btfsc  STATUS, C              ;  Is the LSB Set?
   goto  I2CWHIGH               ;   Yes...

  nop
  bcf    PORTA, 4               ;  Drop the Data Line

  goto   I2CWStrobe             ;  Now, Strobe the Line

I2CWHIGH                        ;  Make the Line high

  bsf    PORTA, 4

  goto   I2CWStrobe

I2CWStrobe                      ;  Strobe the Clock Line

  bsf    PORTA, 3               ;  Make the Line High for 4 usec
  goto   $ + 1
  goto   $ + 1
  bcf    PORTA, 3

  addlw  0 - 1                  ;  Take the Number down by one
  btfss  STATUS, Z              ;  Stop at Zero
   goto  I2CWLoop

  goto   $ + 1                  ;  Now, Receive the Ack

  bsf    PORTA, 4               ;  Make Sure the Data Bus is High for the ACK

  goto   $ + 1
  movf   PORTA, w               ;  Just Strobe the Clock (Nothing Else Changed)
  iorlw  0x008
  movwf  PORTA
  goto   $ + 1                  ;  NOTE: Ack Line is not Checked
  goto   $ + 1
  andlw  0x0F7
  movwf  PORTA                  ;  And, Bring the Line Low

  return                        ;  Return to the Caller


I2CReceive                      ;  Receive a Data Byte

  movwf  I2CTemp                ;  Save the Ack Bit in "w"

  movlw  8                      ;  Sending 8 Bits

  rrf    I2CTemp                ;  Load the Carry Flag with the Ack Bit

I2CRLoop                        ;  Loop Around Here until Each Bit Read

  rrf    I2CTemp                ;  Shift the Data Down

  iorlw  0                      ;  Have we done all 8 bits?
  btfsc  STATUS, Z
   goto  I2CR_Ack               ;  Yes, Carry has the ACK Bit

  addlw  -1                     ;  Decrement the Count

  bsf    PORTA, 4               ;  Make Sure RA4 is controlled by I2C Device
  bsf    PORTA, 3               ;   Strobe the Clock

  nop                           ;  Keep Constant Timing

  bcf    STATUS, C              ;  Carry to be used for the Data
  btfsc  PORTA, 4               ;  Skip the Carry Set if the Line is Low
   bsf   STATUS, C

  bcf    PORTA, 3               ;  Drop the Clock Line

  goto   I2CRLoop               ;  Shift the Carry In and Check the Next Value

I2CR_Ack                        ;  Now, Send the Specified Acknowledge Bit

  bsf    PORTA, 4               ;  Assume there's no ACK
  btfss  STATUS, C
   bcf   PORTA, 4               ;   Or, there is

  bsf    PORTA, 3               ;  Strobe out the Acknowledge

  goto   $ + 1
  goto   $ + 1

  bcf    PORTA, 3

  return                        ;  Return to the Caller
