 subtitle "LCD3LINE - Control an LCD using 3 Lines."
;
;  These routines will Initialize, Send Instructions, and Send Characters to a
;   standard Hitachi 44780 controlled LCD.  A three line interface is used to
;   reduce the number of PIC Pins used on the LCD.
;
;  NOTE:  Before the Initialization is to be executed by the PIC, 15 msecs must
;   elapse before the "Init" subroutine is called.
;
;  The Hardware is:
;
;           |                                   Gnd ----|  1   44780
;   PIC     |                                   Vcc ----|  2   Controlled
;           |                            Contrast V ----|  3   LCD
;           |                              +------------|  4
;           |                              |    Gnd ----|  5
;           |      +---------------------- | -----------|  6
;           |      |                       |            |  7
;           |      |          _______      |            |  8
;       RB7 |------+         | Shift |-----+            |  9
;           |                | Reg   |---------------+  | 10
;           |                |       |-----------+   +--| 11
;       RB6 |----------------| Clk   |-------+   +------| 12
;       RB5 |----------------|_D_____|---+   +----------| 13
;           |                            +--------------| 14
;
;  Hardware Notes:
;   The Pic speed is immaterial
;   RB5 is Connected to Data Out to the LCD Control
;   RB6 is Connected to the LCD Shift Register Clock
;   RB7 is Connected to 'E' Bit of the LCD
;   The Shift Register can be any 5 bit (or more) Serial In/Parallel Out Shift Register
;   LCDTemp, LCDNTemp, LCDCount  - file Registers Used for Data Transfer
;
;  External Routines Required:
;   Dlay5ms - Wait 5 ms
;   Dlay160 - Wait 160 usec for the instruction to complete
;
;  Myke Predko
;  97.01.23
;

LCDInit                         ;  Initialize the LCD

  movlw  3                      ;  Start with the Initialization
  bcf    PORTB, 5               ;  Make sure the RS Flag = 0
  call   LCDNybble              ;  Output the Nybble

  call   Dlay5ms                ;  Wait for the LCD to Power Up

  movlw  3                      ;  Continue with the Reset
  bcf    PORTB, 5
  call   LCDNybble
  call   Dlay160                ;  Wait 160 usec

  movlw  3                      ;  Reset Again
  bcf    PORTB, 5
  call   LCDNybble
  call   Dlay160

  movlw  2                      ;  Now, Set Interface Length to 4 Bits
  bcf    PORTB, 5
  call   LCDNybble
  call   Dlay160

  movlw  0x028
  call   SendINS                ;  Now, Can Just Send Instructions - Set 4 Bits, 2 Lines
  movlw  0x00C                  ;  Turn Display On
  call   SendINS
  movlw  0x001                  ;  Clear the Display, setup the Cursor
  call   SendINS
  movlw  0x006                  ;  Set the Entry Mode

  return


LCDINS                          ;  Send the Byte as an Instruction

  movwf  LCDTemp

  bcf    INTCON, GIE

  bcf    PORTB, 5               ;  Start with the R/S Line Low

  swapf  LCDTemp, w             ;  Get the High Nybble to Send
  call   LCDNybble

  bcf    PORTB, 5               ;  Now, Send the low Nybble

  movf   LCDTemp, w
  call   LCDNybble

  bsf    INTCON, GIE

  call   Dlay160                ;  Wait for the LCD to Process the Instruction

  movlw  0x0FC                  ;  "Clear Display" and "Cursor At Home" Instructions
  andwf  LCDTemp, w             ;   Require 5 msec Delay to Complete
  btfsc  STATUS, Z
   call  Dlay5ms

  return


LCDCHAR                         ;  Send the Byte as Character Data

  movwf  LCDTemp                ;  Save the Value for the Second Nybble

  bcf    INTCON, GIE            ;  Turn OFF Interrupts During Write

  bsf    PORTB, 5               ;  Start with the R/S Line Low

  swapf  LCDTemp, w             ;  Get the High Nybble to Send
  call   LCDNybble

  bsf    PORTB, 5               ;  Now, Send the low Nybble

  movf   LCDTemp, w
  call   LCDNybble

  bsf    INTCON, GIE

  call   Dlay160                ;  Wait for the LCD to Display the Character

  return


LCDNybble                       ;  Send the Nybble (with what's set in PORTB.5 to Begin)

  movwf  LCDNTemp               ;  Save the 4 Bits to Display

  bcf    PORTB, 6               ;  Clock out the Contents of PORTB.5
  bsf    PORTB, 6

  movlw  4                      ;  Now, Shift out four more bits
  movwf  LCDCount

LNLoop                          ;  Loop Around Here

  bcf    PORTB, 5
  rrf    LCDNTemp               ;  Load Carry with the LSB
  btfsc  STATUS, C
   bsf   PORTB, 5               ;  Set the Bit (if Appropriate)

  bcf    PORTB, 6               ;  Clock out the Bit
  bsf    PORTB, 6

  decfsz LCDCount
   goto  LNLoop

  bsf    PORTB, 7               ;  Toggle the 'E' Clock
  bcf    PORTB, 7

  return
